
import "Turbine";
import "Turbine.Gameplay";
import "Turbine.UI";
import "Turbine.UI.Extensions";
import "Turbine.UI.Lotro";

TheOneBagWindow = class( Turbine.UI.Extensions.Window );

function TheOneBagWindow:Constructor()
	Turbine.UI.Extensions.Window.Constructor( self );

	-- Padding used to position the bag initially from the bottom
	-- right of the screen.
	local edgePadding = 50;

	self:SetSize( 215, 600 );
	self:SetBackColor( Turbine.UI.Color() );
	self:SetPosition( Turbine.UI.Display.GetWidth() - self:GetWidth() - edgePadding, Turbine.UI.Display.GetHeight() - self:GetHeight() - edgePadding * 1.5 );
	self:SetText( "GrimBag" );
	self.order = "asc"

	self:SetWantsKeyEvents( true );

	--Hide or show UI from key press
	self.KeyDown = function( sender, args )
		if ( args.Action == Turbine.UI.Lotro.Action.Escape ) then
			sender:SetVisible( false ) 
		end

		if ( args.Action == Turbine.UI.Lotro.Action.ToggleBags or
		     args.Action == Turbine.UI.Lotro.Action.ToggleBag1 or
			 args.Action == Turbine.UI.Lotro.Action.ToggleBag2 or
			 args.Action == Turbine.UI.Lotro.Action.ToggleBag3 or
			 args.Action == Turbine.UI.Lotro.Action.ToggleBag4 or
			 args.Action == Turbine.UI.Lotro.Action.ToggleBag5 or
			 args.Action == Turbine.UI.Lotro.Action.ToggleBag6 )
		then
			sender:SetVisible( not sender:IsVisible() ) 
		end
	end
	
	--Create icons to toggle between ascending and descending order
	local width, height = self:GetSize();
	self.asc_label = Turbine.UI.Label();
	self.asc_label:SetParent( self);
	self.asc_label:SetBlendMode( Turbine.UI.BlendMode.Overlay );
	self.asc_label:SetBackground(0x4112C11C);
	self.asc_label:SetZOrder(10);
	self.asc_label:SetSize(19,15);
	self.desc_label = Turbine.UI.Label();
	self.desc_label:SetParent( self);
	self.desc_label:SetBlendMode( Turbine.UI.BlendMode.Overlay );
	self.desc_label:SetBackground(0x4112C11E);
	self.desc_label:SetZOrder(10);
	self.desc_label:SetSize(19,15);

	local player = Turbine.Gameplay.LocalPlayer.GetInstance();
	self.backpack = player:GetBackpack();
	local theOneBag = self;
	local mainWindow = self;

	self.itemListBoxScrollBar = Turbine.UI.Lotro.ScrollBar();
	self.itemListBoxScrollBar:SetOrientation( Turbine.UI.Orientation.Vertical );
	self.itemListBoxScrollBar:SetParent( self );

	self.itemListBox = Turbine.UI.ListBox();
	self.itemListBox:SetParent( self );
	self.itemListBox:SetOrientation( Turbine.UI.Orientation.Horizontal );
	self.itemListBox:SetVerticalScrollBar( self.itemListBoxScrollBar );
	self.itemListBox:SetAllowDrop( true );
	--Explicitly set colour to make it solid, not semi-transparant
	self.itemListBox:SetBackColor( Turbine.UI.Color(0,0,0) );
	
	--Move items
	self.itemListBox.DragDrop = function( sender, args )
		local shortcut = args.DragDropInfo:GetShortcut();

		if ( shortcut ~= nil ) then
			local item = self.itemListBox:GetItemAt( args.X, args.Y );
			local index = self.itemListBox:IndexOfItem( item );
			
			--Items are moved using the index of the slots inside itemListBox. But when we have the order reversed, bag slot #131 will have index #1 inside the itemListBox (when using 5 items per row). So we need to calculate the proper value.
			if (self.order == "desc") then
				--First, calculate which row it's in.
				if(math.floor(index/self.itemsPerRow) ~= index/self.itemsPerRow) then
					row = math.floor(index/self.itemsPerRow+1);
				else
					row = index/self.itemsPerRow;
				end
				--Turbine.Shell.WriteLine(row)
				--Calculate which bag slot it really is.
				index = self.itemsPerRow*(#self.items/self.itemsPerRow)-(self.itemsPerRow*(row))+index-(self.itemsPerRow*(row-1))
			end
			--Turbine.Shell.WriteLine(index)
			self.backpack:PerformShortcutDrop( shortcut, index, Turbine.UI.Control.IsShiftKeyDown() );
		end
	end

	--Resize window
	self.resizeHandle = Turbine.UI.Control();
	self.resizeHandle:SetParent( self );
	self.resizeHandle:SetZOrder( 100 );
	self.resizeHandle:SetSize( 20, 20 );
	self.resizeHandle:SetPosition( self:GetWidth() - self.resizeHandle:GetWidth(), self:GetHeight() - self.resizeHandle:GetHeight() );

	self.resizeHandle.MouseDown = function( sender, args )
		sender.dragStartX = args.X;
		sender.dragStartY = args.Y;
		sender.dragging = true;
	end

	self.resizeHandle.MouseMove = function( sender, args )
		local width, height = mainWindow:GetSize();

		if ( sender.dragging ) then
			--Not allowing changing of width
			--mainWindow:SetSize( width + ( args.X - sender.dragStartX ), height + ( args.Y - sender.dragStartY ) );
			mainWindow:SetSize( width, height + ( args.Y - sender.dragStartY ) );
			sender:SetPosition( mainWindow:GetWidth() - sender:GetWidth(), mainWindow:GetHeight() - sender:GetHeight() )
			theOneBag:PerformLayout()
		end
	end

	self.resizeHandle.MouseUp = function( sender, args )
		sender.dragging = false;
	end

	-- Initialize the container that holds the elements referencing
	-- the player's inventory.
	self.items = { };

	self.backpack.SizeChanged = function( sender, args )
		theOneBag:Refresh();
	end

	self.backpack.ItemAdded = function( sender, args )
		theOneBag.items[args.Index]:SetItem( theOneBag.backpack:GetItem( args.Index ) );
	end

	self.backpack.ItemRemoved = function( sender, args )
		--Line below did not work when item was removed by equipping, for some reason. Maybe some delay between remove-event fires and the items is actually equipped? No idea... Anyway, seem to work to just set the content of the slot to nothing, which should be correct if the item was removed.
		--theOneBag.items[args.Index]:SetItem( theOneBag.backpack:GetItem( args.Index ) );
		theOneBag.items[args.Index]:SetItem( nil );
	end

	self.backpack.ItemMoved = function( sender, args )
		theOneBag.items[args.OldIndex]:SetItem( theOneBag.backpack:GetItem( args.OldIndex ) );
		theOneBag.items[args.NewIndex]:SetItem( theOneBag.backpack:GetItem( args.NewIndex ) );
	end
	self.desc_label.MouseClick = function( sender, args )
		self.order = "desc"
		self:Refresh();
	end
	self.asc_label.MouseClick = function( sender, args )
		self.order = "asc"
		self:Refresh();
	end


	self:Refresh();
end

function TheOneBagWindow:Refresh()
	local backpackSize = self.backpack:GetSize();
	local rowitems = {};
	--Loop through every item in the backpack
	self.itemListBox:ClearItems();
	for i = 1, backpackSize, 1 do
		if ( self.items[i] ) then
			self.items[i]:SetParent( nil );
		end
		self.items[i] = Turbine.UI.Lotro.ItemControl( self.backpack:GetItem( i ) );
		self.items[i]:SetBackground( 0x411386AF );
		--self.items[i]:SetStretchMode(1);
		self.items[i]:SetBlendMode( Turbine.UI.BlendMode.Overlay );
		--Create label with slot number
		label = Turbine.UI.Label();
		label:SetParent( self.items[i] );
		label:SetText(i);
		label:SetPosition( 8, 8 );
		--Add some shadow around the text to make it easier to read
		label:SetOutlineColor( Turbine.UI.Color( 0, 0, 0 ) );
		label:SetFontStyle( Turbine.UI.FontStyle.Outline );
		--Put it in front
		label:SetZOrder(9000);
		--Make it non-clickable (or you'll click on the label when you want to click on the item behind
		label:SetMouseVisible(false);
		--Calculate which row this slot belongs to. 1 row = 5 slots. Starts with row 1, not 0.
		if(math.floor(i/5) ~= i/5) then
			row = math.floor(i/5+1);
		else
			row = i/5;
		end
		--Create array if it does not exist. Even if rowitems exist you can't create rowitems[2][5] if rowitems[2] is not set.
		if(rowitems[row] == nil) then
			rowitems[row] = {};
		end
		
		--Add the itemlabel to rowitems. Second key is the number (1-5) the slot is in this row. Ex. slot #5 is [1][5] and slot #6 is [2][1]
		rowitems[row][i-5*(row-1)] = self.items[i];
		
	end

	--Sort the rows in reverse order. First tried this with SetReverseFill in different ways, but it insisted on starting filling from bottom of visible window instead of bottom of list.
	--We're not reversing the order like:
	--10 9 8 7 6
	--5 4 3 2 1
	--But rather
	--6 7 8 9 10
	--1 2 3 4 5
	--Just the rows are reversed, not the slots within these rows.
	--These are the reason we use rowitems at all, rather than just add to itemListBox directly
	if (self.order == "desc") then
		local i, j = 1, #rowitems
		while i < j do
			rowitems[i], rowitems[j] = rowitems[j], rowitems[i]
			i = i + 1
			j = j - 1
		end
	end
	--Now that the row is sorted, we loop through it, first loop through the row
	for i, row in ipairs(rowitems) do
		--Loop through each slot and add to the UI
		for j, item in ipairs(rowitems[i]) do
			self.itemListBox:AddItem( rowitems[i][j]);
		end
	end
	self:PerformLayout();
end

function TheOneBagWindow:PerformLayout()
	self:Layout( { } )
end

function TheOneBagWindow:Layout( args )
	local width, height = self:GetSize();
	
	local itemWidth = 40;
	
	if ( self.items[1] ~= nil ) then
		itemWidth = self.items[1]:GetWidth()
	end

	local listWidth = width - 40;
	local listHeight = height - 50;
	self.itemsPerRow = listWidth / itemWidth;

	self.itemListBox:SetPosition( 15, 35 );
	self.itemListBox:SetSize( listWidth, listHeight );
	self.itemListBox:SetMaxItemsPerLine( self.itemsPerRow );

	self.itemListBoxScrollBar:SetPosition( width - 20, 35 );
	self.itemListBoxScrollBar:SetSize( 10, listHeight );
	self.itemListBoxScrollBar:SetZOrder(9001);
	self.asc_label:SetPosition( width-60, 22 );
	self.desc_label:SetPosition( width-40, 20 );
end
